from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.testns.testcoll.plugins.action.echoaction import ActionModule as BaseAM


class ActionModule(BaseAM):
    pass
